<?php

namespace Drupal\drupalmoduleupgrader;

/**
 * Interface for plugins which can analyze a target module and flag potential
 * or existing issues.
 */
interface AnalyzerInterface {

  /**
   * Error level for issues which will cause fatal errors and/or exceptions
   * in Drupal 8. Show stoppers, basically.
   */
  const ERROR = 'error';

  /**
   * Error level for issues which will affect (and probably break) module
   * functionality, but not cause fatal errors or exceptions.
   */
  const WARNING = 'warning';

  /**
   * Analyzes a target module and flags any issues found.
   *
   * @param TargetInterface $target
   *  The target module.
   *
   * @return IssueInterface|IssueInterface[]|NULL
   *  A single issue, an array of issues, or NULL to indicate that no issues
   *  were found.
   */
  public function analyze(TargetInterface $target);

}
