<?php

namespace Drupal\drupalmoduleupgrader\Plugin\DMU\Converter\Functions;

use Drupal\Core\Plugin\ContainerFactoryPluginInterface;
use Drupal\Core\StringTranslation\TranslationInterface;
use Drupal\drupalmoduleupgrader\TargetInterface;
use Pharborist\Types\ArrayNode;
use Pharborist\Objects\ClassMethodCallNode;
use Pharborist\Functions\FunctionCallNode;
use Pharborist\LineCommentBlockNode;
use Symfony\Component\DependencyInjection\ContainerInterface;

/**
 * @Converter(
 *  id = "entity_load",
 *  description = @Translation("Rewrites calls to entity_load().")
 * )
 */
class EntityLoad extends FunctionCallModifier implements ContainerFactoryPluginInterface {

  public function __construct(array $configuration, $plugin_id, $plugin_definition, TranslationInterface $translator) {
    $this->stringTranslation = $translator;
  }

  /**
   * {@inheritdoc}
   */
  public static function create(ContainerInterface $container, array $configuration, $plugin_id, $plugin_definition) {
    return new static(
      $configuration,
      $plugin_id,
      $plugin_definition,
      $container->get('string_translation')
    );
  }

  /**
   * {@inheritdoc}
   */
  public function rewrite(FunctionCallNode $call, TargetInterface $target) {
    $arguments = $call->getArguments();

    // If there were three arguments, the call is affecting the internal
    // entity cache. Unfortunately, it's pretty much impossible to reliably
    // determine whether or not they wanted to reset the cache, so let's just
    // leave a FIXME.
    if (sizeof($arguments) == 3) {
      LineCommentBlockNode::create($this->t('FIXME: To reset the entity cache, use EntityStorageInterface::resetCache().'))->insertBefore($call);
    }

    $rewritten = ClassMethodCallNode::create('\Drupal', 'entityManager')
      ->appendMethodCall('getStorage')
      ->appendArgument(clone $arguments[0]);

    // If there's a third argument, conditions were passed. Not a recommended
    // practice, but certain modules might have done it anyway. If we detect
    // conditions, use loadByProperties().
    if (sizeof($arguments) > 2) {
      return $rewritten
        ->appendMethodCall('loadByProperties')
        ->appendArgument(clone $arguments[2]);
    }
    else {
      $rewritten->appendMethodCall('load');

      if (sizeof($arguments) > 1 && $arguments[1] instanceof ArrayNode) {
        $rewritten->appendArgument(clone $arguments[1]);
      }

      return $rewritten;
    }
  }

}
