<?php

namespace Drupal\drupalmoduleupgrader\Plugin\DMU\Converter\Functions;

use Drupal\Core\Plugin\ContainerFactoryPluginInterface;
use Drupal\Core\StringTranslation\TranslationInterface;
use Drupal\drupalmoduleupgrader\TargetInterface;
use Pharborist\Objects\ClassMethodCallNode;
use Pharborist\Functions\FunctionCallNode;
use Pharborist\LineCommentBlockNode;
use Pharborist\Types\ScalarNode;
use Pharborist\Types\StringNode;
use Symfony\Component\DependencyInjection\ContainerInterface;
use Symfony\Component\Yaml\Yaml;

/**
 * Parent class of the VariableGet, VariableSet, and VariableDel plugins.
 */
abstract class VariableAPI extends FunctionCallModifier implements ContainerFactoryPluginInterface {

  public function __construct(array $configuration, $plugin_id, $plugin_definition, TranslationInterface $translator) {
    parent::__construct($configuration, $plugin_id, $plugin_definition);
    $this->stringTranslation = $translator;
  }

  /**
   * {@inheritdoc}
   */
  public static function create(ContainerInterface $container, array $configuration, $plugin_id, $plugin_definition) {
    return new static(
      $configuration,
      $plugin_id,
      $plugin_definition,
      $container->get('string_translation')
    );
  }

  /**
   * Helper for subclasses' rewrite() methods. This checks if the call can
   * be rewritten at all and leaves a FIXME if it can't. If the variable's
   * key is not a string starting with MODULE_, the call will not be
   * considered rewritable.
   *
   * @return boolean
   */
  protected function tryRewrite(FunctionCallNode $call, TargetInterface $target) {
    $statement = $call->getStatement();
    $arguments = $call->getArguments();

    if ($arguments[0] instanceof StringNode) {
      $key = $arguments[0]->toValue();

      if (strPos($key, $target->id() . '_') === 0) {
        return TRUE;
      }
      else {
        $comment = <<<END
This looks like another module's variable. You'll need to rewrite this call
to ensure that it uses the correct configuration object.
END;
        LineCommentBlockNode::create($this->t($comment))->prependTo($statement);
        return FALSE;
      }
    }
    else {
      $comment = <<<END
The correct configuration object could not be determined. You'll need to
rewrite this call manually.
END;
      LineCommentBlockNode::create($this->t($comment))->prependTo($statement);
      return FALSE;
    }
  }

}
