<?php

namespace Drupal\drupalmoduleupgrader\Plugin\DMU\Converter;

use Drupal\Core\Plugin\ContainerFactoryPluginInterface;
use Drupal\drupalmoduleupgrader\ConverterBase;
use Drupal\drupalmoduleupgrader\TargetInterface;
use Symfony\Component\DependencyInjection\ContainerInterface;

/**
 * @Converter(
 *  id = "grep",
 *  description = @Translation("Searches for and replaces commonly-used code that has changed in Drupal 8.")
 * )
 */
class Grep extends ConverterBase implements ContainerFactoryPluginInterface {

  /**
   * @var string[]
   */
  private $targets = [];

  public function __construct(array $configuration, $plugin_id, $plugin_definition, array $config) {
    parent::__construct($configuration, $plugin_id, $plugin_definition);

    foreach ($config['globals'] as $variable => $replacement) {
      $this->targets['global $' . $variable] = '$' . $variable . ' = ' . $replacement;
      $this->targets['$GLOBALS[\'' . $variable . '\']'] = $replacement;
      $this->targets['$GLOBALS["' . $variable . '"]'] = $replacement;
    }
    foreach ($config['function_calls'] as $call => $replacement) {
      $this->targets[$call . '('] = $replacement . '(';
    }
    foreach ($config['constants'] as $constant => $replacement) {
      $this->targets[$constant] = $replacement;
    }
  }

  /**
   * {@inheritdoc}
   */
  public static function create(ContainerInterface $container, array $configuration, $plugin_id, $plugin_definition) {
    return new static(
      $configuration,
      $plugin_id,
      $plugin_definition,
      $container->get('config.factory')->get('drupalmoduleupgrader.grep')->get()
    );
  }

  /**
   * {@inheritdoc}
   */
  public function convert(TargetInterface $target) {
    // Flush other open syntax trees to ensure that other plugins don't clobber
    // our changes later.
    $target->getCodeManager()->flush();

    foreach ($target->getFinder() as $file) {
      // Load in the entire contents of the module. This is criminally inefficient
      // and wasteful of memory and should eventually be refactored into something
      // a little more...I dunno, sustainable.
      /** @var \Symfony\Component\Finder\SplFileInfo $file */
      $search = array_keys($this->targets);
      $replace = array_values($this->targets);
      file_put_contents($file->getPathname(), str_replace($search, $replace, $file->getContents()));
    }
  }

}
