<?php

namespace Drupal\drupalmoduleupgrader\Plugin\DMU\Converter;

use Drupal\Core\Plugin\ContainerFactoryPluginInterface;
use Drupal\drupalmoduleupgrader\ConverterBase;
use Drupal\drupalmoduleupgrader\TargetInterface;
use Drupal\drupalmoduleupgrader\Utility\StringTransformTrait;
use Psr\Log\LoggerInterface;
use Symfony\Component\DependencyInjection\ContainerInterface;

/**
 * @Converter(
 *  id = "hook_field_widget_info",
 *  description = @Translation("Creates formatter class templates from hook_field_widget_info()."),
 *  hook = "field_widget_info"
 * )
 */
class HookFieldWidgetInfo extends ConverterBase implements ContainerFactoryPluginInterface {

  use StringTransformTrait;

  /**
   * @var \Psr\Log\LoggerInterface
   */
  protected $logger;

  public function __construct(array $configuration, $plugin_id, $plugin_definition, LoggerInterface $logger) {
    parent::__construct($configuration, $plugin_id, $plugin_definition);
    $this->logger = $logger;
  }

  /**
   * {@inheritdoc}
   */
  public static function create(ContainerInterface $container, array $configuration, $plugin_id, $plugin_definition) {
    return new static(
      $configuration,
      $plugin_id,
      $plugin_definition,
      $container->get('logger.factory')->get('default')
    );
  }

  /**
   * {@inheritdoc}
   */
  public function convert(TargetInterface $target) {
    try {
      $widgets = $this->executeHook($target, $this->pluginDefinition['hook']);
    }
    catch (\LogicException $e) {
      $this->logger->warning($e->getMessage(), [
        'target' => $target->id(),
        'hook' => $this->pluginDefinition['hook'],
      ]);
      return;
    }

    foreach ($widgets as $id => $widget) {
      $render = [
        '#module' => $target->id(),
        '#class' => $this->toTitleCase($id),
        '#theme' => 'dmu_widget',
        '#info' => [
          'id' => $id,
          'label' => $widget['label'],
          'description' => $widget['description'] ?: NULL,
          'field_types' => $widget['field types'],
        ],
      ];
      $this->writeClass($target, $this->parse($render));
    }
  }

}
