<?php

namespace Drupal\drupalmoduleupgrader\Plugin\DMU\Converter;

use Drupal\Component\Plugin\PluginManagerInterface;
use Drupal\Core\Plugin\ContainerFactoryPluginInterface;
use Drupal\drupalmoduleupgrader\ConverterBase;
use Drupal\drupalmoduleupgrader\TargetInterface;
use Drupal\drupalmoduleupgrader\Utility\Filter\FunctionCallArgumentFilter;
use Pharborist\Functions\FunctionCallNode;
use Pharborist\Functions\FunctionDeclarationNode;
use Pharborist\Functions\ParameterNode;
use Symfony\Component\DependencyInjection\ContainerInterface;

/**
 * @Converter(
 *  id = "hook_node_prepare",
 *  description = @Translation("Converts hook_node_prepare() into hook_ENTITY_TYPE_prepare_form()."),
 *  hook = "node_prepare"
 * )
 */
class HookNodePrepare extends ConverterBase implements ContainerFactoryPluginInterface {

  /**
   * @var \Drupal\Component\Plugin\PluginManagerInterface
   */
  protected $rewriters;

  public function __construct(array $configuration, $plugin_id, $plugin_definition, PluginManagerInterface $rewriters) {
    parent::__construct($configuration, $plugin_id, $plugin_definition);
    $this->rewriters = $rewriters;
  }

  /**
   * {@inheritdoc}
   */
  public static function create(ContainerInterface $container, array $configuration, $plugin_id, $plugin_definition) {
    return new static(
      $configuration,
      $plugin_id,
      $plugin_definition,
      $container->get('plugin.manager.drupalmoduleupgrader.rewriter')
    );
  }

  public function convert(TargetInterface $target) {
    /** @var \Pharborist\Functions\FunctionDeclarationNode $function */
    $function = $target->getIndexer('hook')->get('node_prepare')->get(0);

    // foo_node_prepare() --> foo_node_prepare_form().
    $function->setName($function->getName() . '_form');

    // The first parameter is a node, so rewrite the function accordingly.
    $this->rewriters
      ->createInstance('_entity:node')
      ->rewrite($function->getParameterAtIndex(0));

    // Create the $operation parameter.
    $function->appendParameter(ParameterNode::create('operation'));

    // Create the $form_state parameter.
    $form_state = ParameterNode::create('form_state')->setTypeHint('\Drupal\Core\Form\FormStateInterface');
    $function->appendParameter($form_state);

    $target->getCodeManager()->save($function);
  }

}
